class Toolbox (object):
    def __init__(self):
        """Define the toolbox (the name of the toolbox is the name of the .pyt file)."""
        self.label = "Email Toolbox For Pro"
        self.alias = "email"

        # List of tool classes associated with this toolbox
        self.tools = [EmailTool]


class EmailTool(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Send Email"
        self.description = "This tool allows you to email your file using public Email as a SMTP server"
        self.canRunInBackground = False

    def getParameterInfo(self):
        """Define parameter definitions"""

        sendto = arcpy.Parameter(
            displayName="To",
            name="To",
            datatype="String",
            parameterType="Required",
            direction="Input",
            multiValue=True)

        fromaddr = arcpy.Parameter(
            displayName="From",
            name="From",
            datatype="String",
            parameterType="Required",
            direction="Input")

        subject = arcpy.Parameter(
            displayName="Subject",
            name="Subject",
            datatype="String",
            parameterType="Required",
            direction="Input")

        text = arcpy.Parameter(
            displayName="Text",
            name="Text",
            datatype="String",
            parameterType="Required",
            direction="Input")

        attachedfile = arcpy.Parameter(
            displayName="Attach File",
            name="Attach_File",
            datatype="File",
            parameterType="Optional",
            direction="Input",
            multiValue=True)

        smtpMailServer = arcpy.Parameter(
            displayName="SMTP Email Server",
            name="SMTP_Server",
            datatype="String",
            parameterType="Required",
            direction="Input")

        portNumber = arcpy.Parameter(
            displayName="Port Number",
            name="Port_Number",
            datatype="String",
            parameterType="optional",
            direction="Input")

        smtpUser = arcpy.Parameter(
            displayName="User",
            name="User",
            datatype="String",
            parameterType="optional",
            direction="Input")

        smtpPwd =  arcpy.Parameter(
            displayName="Password",
            name="Password",
            datatype="GPStringHidden",
            parameterType="optional",
            direction="Input")

        sent = arcpy.Parameter(
            displayName="Sent",
            name="Sent",
            datatype="String",
            parameterType="derived",
            direction="Output")

        params = [sendto, fromaddr, subject, text, attachedfile, smtpMailServer, portNumber,  smtpUser, smtpPwd, sent]

        return params

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def send_mail(self, send_from, send_to, subject, text, attachedfiles, server, portNumber = "", smtpUser="", smtpPwd=""):
        from email.message import EmailMessage
        from email.headerregistry import Address
        from email.utils import make_msgid
        import arcpy
        import os
        import smtplib
        import mimetypes
        
        
        msg = EmailMessage()
        msg['From'] = Address(addr_spec=send_from)
        ToList = (Address(addr_spec=x) for x in send_to)
        msg['To'] = ToList

        msg['Subject'] = subject
    
        msg.set_content(text)
        
        if attachedfiles:
            for path in attachedfiles:
                path = str(path)
                # Guess the content type based on the file's extension.  Encoding
                # will be ignored, although we should check for simple things like
                # gzip'd or compressed files.
                ctype, encoding = mimetypes.guess_type(path)
                if ctype is None or encoding is not None:
                    # No guess could be made, or the file is encoded (compressed), so
                    # use a generic bag-of-bits type.
                    ctype = 'application/octet-stream'
                maintype, subtype = ctype.split('/', 1)
                with open(path, 'rb') as fp:
                    msg.add_attachment(fp.read(),
                                       maintype=maintype,
                                       subtype=subtype,
                                       filename=os.path.basename(path))                        
              
        with smtplib.SMTP(server, portNumber) as smtp:           
            smtp.ehlo()
            smtp.starttls()
            smtp.ehlo()
            smtp.login(smtpUser, smtpPwd)                  
            smtp.send_message(msg)

    def execute(self, parameters, messages):
    
        """The source code of the tool."""
        
        sendto = parameters[0].values
        fromaddr = parameters[1].valueAsText
        subject = parameters[2].valueAsText
        text = parameters[3].valueAsText
        attachedfiles = parameters[4].values
        smtpMailServer = parameters[5].valueAsText
        portNumber = parameters[6].valueAsText
        smtpUser = parameters[7].valueAsText
        smtpPwd = parameters[8].valueAsText

        self.send_mail(fromaddr, sendto, subject, text, attachedfiles, smtpMailServer, portNumber, smtpUser, smtpPwd)
        return